@@ -129,6 +129,14 @@ static struct i2c_adap *more_adapters(struct i2c_adap *adapters, int n)
 	return new_adapters;
 }
 
+static int sort_i2c_busses(const void *a, const void *b)
+{
+	const struct i2c_adap *adap1 = a;
+	const struct i2c_adap *adap2 = b;
+
+	return adap1->nr - adap2->nr;
+}
+
 struct i2c_adap *gather_i2c_busses(void)
 {
 	char s[120];
@@ -314,6 +322,9 @@ found:
 	closedir(dir);
 
 done:
+	/* Sort by bus number for convenience */
+	qsort(adapters, count, sizeof(struct i2c_adap), sort_i2c_busses);
+
 	return adapters;
 }
 
@@ -381,7 +392,7 @@ int parse_i2c_address(const char *address_arg, int all_addrs)
 {
 	long address;
 	char *end;
-	long min_addr = 0x03;
+	long min_addr = 0x08;
 	long max_addr = 0x77;
 
 	address = strtol(address_arg, &end, 0);
@@ -406,14 +417,21 @@ int parse_i2c_address(const char *address_arg, int all_addrs)
 
 int open_i2c_dev(int i2cbus, char *filename, size_t size, int quiet)
 {
-	int file;
+	int file, len;
 
-	snprintf(filename, size, "/dev/i2c/%d", i2cbus);
-	filename[size - 1] = '\0';
+	len = snprintf(filename, size, "/dev/i2c/%d", i2cbus);
+	if (len >= (int)size) {
+		fprintf(stderr, "%s: path truncated\n", filename);
+		return -EOVERFLOW;
+	}
 	file = open(filename, O_RDWR);
 
 	if (file < 0 && (errno == ENOENT || errno == ENOTDIR)) {
-		sprintf(filename, "/dev/i2c-%d", i2cbus);
+		len = snprintf(filename, size, "/dev/i2c-%d", i2cbus);
+		if (len >= (int)size) {
+			fprintf(stderr, "%s: path truncated\n", filename);
+			return -EOVERFLOW;
+		}
 		file = open(filename, O_RDWR);
 	}
 
